#include <cstring>
#include <iostream>

#include <netplus/exception.h>
#include <netplus/socket.h>

#include <httppp/http.h>
#include <httppp/httpd.h>
#include <httppp/exception.h>

#include <unistd.h>
#include <pwd.h>
#include <grp.h>
#include <dirent.h>
#include <sys/stat.h>

#include <config.h>

class UIFile {

};

class UICache {


};

class Controller : public netplus::event  {
public:
    Controller(netplus::socket *serversocket) :event(serversocket){
        srvSock=serversocket;
        admSock = new netplus::tcp(ADMINCTL_BIND_DIR(adm.sock),1,0);
    };

    ~Controller(){
        delete admSock;
    }

    void RequestEvent(netplus::con *curcon){
        libhttppp::HttpRequest http;
        try{
            http.parse(curcon);
            if(strncmp(http.getRequestURL(),"/admin/api",10)>=0){
                try{
                    netplus::tcp *csock=admSock->connect();
                    std::string header;
                    http.printHeader(header);
                    std::cout << header <<std::endl;
                    admSock->sendData(csock,(void*)header.c_str(),header.length());
                    char data[16384];

                    int dlen=0;

                    do{
                        dlen=admSock->recvData(csock,data,16384);
                    }while(dlen==0);

                    libhttppp::HttpResponse resp;

                    size_t hsize=resp.parse(data,dlen);
                    size_t recvd=dlen-hsize;

                    header.clear();
                    resp.printHeader(header);
                    srvSock->sendData(curcon->csock,(void*)header.c_str(),header.length());

                    if(recvd>0){
                        recvd+=srvSock->sendData(curcon->csock,data+hsize,recvd);
                    }

                    while(recvd < resp.getContentLength()){
                        dlen=admSock->recvData(csock,data,16384);
                        recvd+=dlen;
                        while((dlen-=srvSock->sendData(curcon->csock,data,dlen))!=0);
                        std::cout << recvd << std::endl;
                    }
                    delete csock;
                }catch(netplus::NetException &e){
                    libhttppp::HttpResponse resp;
                    resp.setVersion(HTTPVERSION(1.1));
                    resp.setState(HTTP504);
                    resp.send(curcon,e.what());
                    return;
                }
            }else if(strncmp(http.getRequestURL(),"/admin",6)>=0){
                libhttppp::HttpResponse resp;
                resp.setVersion(HTTPVERSION(1.1));
                resp.setState(HTTP200);
                resp.setContentType("text/html");
                resp.send(curcon,"index",5);
                return;
            }else{
                libhttppp::HttpResponse resp;
                resp.setVersion(HTTPVERSION(1.1));
                resp.setState(HTTP404);
                resp.setContentType("text/html");
                resp.send(curcon,"Wrong Url",10);
                return;
            }
        }catch(libhttppp::HTTPException &e){
            std::cerr << e.what() << std::endl;
        }

    };
private:
    netplus::socket *srvSock;
    netplus::tcp    *admSock;
};

class  ControllerHttpD : public libhttppp::HttpD {
public:
    ControllerHttpD(const char *httpaddr, int port,int maxconnections,const char *sslcertpath,const char *sslkeypath)
    : HttpD(httpaddr,port,maxconnections,sslcertpath,sslkeypath){
        libhttppp::HTTPException httpexception;
        try {
            Controller ctl(getServerSocket());
            ctl.runEventloop();
        }catch(netplus::NetException &e){
            std::cout << e.what() << std::endl;
        }
    };
};

int main(int argc,char *argv[]){
    DIR *bindDir;

    if(getuid()!=0){
        std::cerr << "This Programm must be run as root !" << std::endl;
        return -1;
    }

    struct group *grp = getgrnam(SSOD_GROUP);
    if(!grp)
        return -1;

    struct passwd *pwd = getpwnam(SSOD_USER);
    if(!pwd)
        return -1;

    bindDir = opendir (ADMINCTL_BIND_DIR());
    if (!bindDir){
        mkdir(ADMINCTL_BIND_DIR(),0770);
        chown(ADMINCTL_BIND_DIR(),pwd->pw_gid,grp->gr_gid);
    }else{
        closedir (bindDir);
    }

    pid_t apid=fork();

    if(apid==0){
        char *earg[]={(char*)"ssod-adminctl",nullptr};
        execv(LIBEXEC_DIR(ssod-adminctl),earg);
        return -1;
    }else{
        sleep(2);

        chown(ADMINCTL_BIND_DIR(adm.sock),pwd->pw_gid,grp->gr_gid);
        chmod(ADMINCTL_BIND_DIR(adm.sock),0770);

        setgid(grp->gr_gid);
        setuid(pwd->pw_gid);

        ControllerHttpD ctl("0.0.0.0",8080,1024,nullptr,nullptr);
    }

    return 0;
}
